<?php

use Phinx\Migration\AbstractMigration;
use Phinx\Db\Adapter\MysqlAdapter;

class AddCategorie extends AbstractMigration
{
    public function change()
    {
        $this
            ->table("categorie", ['id' => false, 'primary_key' => ["categorie_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('categorie_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('parent_id', 'integer', ['null' => true, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'categorie_id', 'signed' => false])
            ->addColumn('categorie_rang', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'parent_id'])
            ->addColumn('categorie_date_ajout', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'categorie_rang'])
            ->addColumn('categorie_date_modif', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'categorie_date_ajout'])
            ->addColumn('categorie_visible', 'integer', ['null' => false, 'default' => '1', 'limit' => MysqlAdapter::INT_TINY, 'precision' => 3, 'after' => 'categorie_date_modif'])
            ->addColumn('categorie_image', 'string', ['null' => false, 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'categorie_visible'])
            ->addColumn('categorie_ext_id', 'string', ['null' => false, 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'categorie_image'])
            ->addColumn('categorie_code', 'char', ['null' => false, 'limit' => 10, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'categorie_ext_id'])
            ->addForeignKey('parent_id', 'categorie', 'categorie_id', array(
                'delete' => 'SET NULL',
                'update' => 'CASCADE'
            ))
            ->addIndex(['parent_id'], ['unique' => false])
            ->addIndex(['categorie_ext_id'], ['unique' => false])
            ->addIndex(['categorie_visible'], ['unique' => false])
            ->addIndex(['categorie_rang'], ['unique' => false])
            ->create();

        $this
            ->table("categorie_boutique", ['id' => false, 'primary_key' => ["categorie_boutique_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('categorie_boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('categorie_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'categorie_boutique_id', 'signed' => false])
            ->addColumn('boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'categorie_id', 'signed' => false])
            ->addColumn('categorie_boutique_statut_id', 'integer', ['null' => false, 'default' => '1', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'boutique_id'])
            ->addIndex(['categorie_id', 'boutique_id'], ['unique' => true])
            ->addIndex(['categorie_id'], ['unique' => false])
            ->addIndex(['boutique_id'], ['unique' => false])
            ->addIndex(['categorie_boutique_statut_id'], ['unique' => false])
            ->addForeignKey('categorie_id', 'categorie', 'categorie_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('boutique_id', 'boutique', 'boutique_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->create();

        $this
            ->table("categorie_nom", ['id' => false, 'primary_key' => ["categorie_nom_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('categorie_nom_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('categorie_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'categorie_nom_id', 'signed' => false])
            ->addColumn('langue_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'categorie_id', 'signed' => false])
            ->addColumn('categorie_nom_nom', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'langue_id'])
            ->addColumn('categorie_nom_desc', 'text', ['null' => false, 'limit' => 65535, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'categorie_nom_nom'])
            ->addColumn('categorie_nom_groupe', 'integer', ['null' => true, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'categorie_nom_desc'])
            ->addColumn('categorie_nom_meta_title', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'categorie_nom_groupe'])
            ->addColumn('categorie_nom_meta_desc', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'categorie_nom_meta_title'])
            ->addColumn('categorie_nom_meta_kw', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'categorie_nom_meta_desc'])
            ->addColumn('categorie_nom_meta_other', 'text', ['null' => false, 'limit' => 65535, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'categorie_nom_meta_kw'])
            ->addForeignKey('categorie_id', 'categorie', 'categorie_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('langue_id', 'langue', 'langue_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addIndex(['langue_id', 'categorie_id'], ['name' => "langue_id_2", 'unique' => true])
            ->addIndex(['langue_id'], ['name' => "langue_id", 'unique' => false])
            ->addIndex(['categorie_id'], ['name' => "categorie_id", 'unique' => false])
            ->create();
    }
}
