<?php

namespace Form;

use Form\Constraint\Callback;
use Form\Constraint\Date;
use Form\Constraint\Mail;
use Form\Constraint\NotBlank;
use Form\Constraint\Phone;
use Form\Constraint\UnequalToField;

class ProfileForm extends Form
{
    function __construct()
    {
        parent::__construct();

        $this
            ->add(
                new Field(
                    "client_email",
                    array(
                        "not_empty"  => new NotBlank(),
                        "is_mail"    => new Mail(),
//                        "dispo_mail" => new Callback(array("Client", "mailIsFree")),
                    ),
                    array(
                        "not_empty"  => "Vous devez remplir le champs email.",
                        "is_mail"    => "Vous devez utiliser un email correct.",
//                        "dispo_mail" => "L'adresse email que vous avez renseigné est déjà utilisée.",
                    )
                )
            )
            ->add(
                new Field(
                    "client_referer",
                    array(
                        "referer"      => new Callback(array("Client", "mailIsUsed")),
                        "referer_diff" => new UnequalToField("client_email"),
                    ),
                    array(
                        "referer"      => "L'adresse du parrain entrée est introuvable.",
                        "referer_diff" => "L'adresse de votre parrain et votre email ne peuvent pas être identiques.",
                    )
                )
            )
            ->add(
                new Field(
                    "client_societe",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => "Vous devez remplir le champ societe.")
                )
            )
            ->add(
                new Field(
                    "civilite_id",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => "Vous devez remplir le champ civilite.")
                )
            )
            ->add(
                new Field(
                    "client_nom",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => "Vous devez remplir le champ Nom.")
                )
            )
            ->add(
                new Field(
                    "client_prenom",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => "Vous devez remplir le champ Prénom.")
                )
            )
            ->add(
                new Field(
                    "adresse_rue",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => "Vous devez remplir le champ rue.")
                )
            )
            ->add(
                new Field(
                    "adresse_cp",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => "Vous devez remplir le champ code postal.")
                )
            )
            ->add(
                new Field(
                    "adresse_ville",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => "Vous devez remplir le champ ville.")
                )
            )
            ->add(
                new Field(
                    "pays_id",
                    array("not_empty" => new NotBlank()),
                    array("not_empty" => "Vous devez choisir un pays.")
                )
            )
            ->add(
                new Field(
                    "adresse_telephone",
                    array(
                        "not_empty" => new NotBlank(),
                        "is_number" => new Phone(),
                    ),
                    array(
                        "not_empty" => "Vous devez remplir le numéro de téléphone.",
                        "is_number" => "La valeur remplie n'est pas un numéro de téléphone.",
                    )
                )
            )
            ->add(
                new Field(
                    "client_ddn",
                    array(
                        "not_empty" => new NotBlank(),
                        "date_ddn"  => new Date("Y-m-d"),
                    ),
                    array(
                        "not_empty" => "Vous devez remplir une date de naissance.",
                        "date_ddn"  => "Date de naissance invalide.",
                    )
                )
            )
        ;
    }

}