<?php

/**
 * Class Model
 *
 * @property SiteSQL $sql
 */
class Model
{

    /**
     * @var array
     */
    protected static $globalI18n = array();
    /**
     * @var array
     */
    protected static $gloabalPrimaryKey = array();
    /**
     * @var string
     */
    protected $suffixI18n = '_nom';
    /**
     * @var bool
     */
    protected $i18n;
    /**
     * @var string
     */
    protected $primaryKey;
    /**
     * @var array
     */
    protected $innerJoin = array();
    /**
     * @var array
     */
    protected $leftJoin = array();
    /**
     * @var array
     */
    protected $where = array();
    /**
     * @var array
     */
    protected $extras = array();
    /**
     * @var array
     */
    protected $properties = array();

    /**
     * Constructeur
     *
     * @param integer|null $id id to load
     *
     * @throws Exception
     */
    public function __construct($id = null)
    {

        if (empty($this->table)) {
            throw new Exception('properties $table is undefined');
        }

        $this->connection(SiteSQL::getInstance());

        $this->i18n = $this->resolveI18nTable();
        $this->primaryKey = $this->resolvePrimaryKey();

        if ($this->i18n) {
            $tableI18n = $this->table . $this->suffixI18n;
            $alias = Query::getAlias($this->table);
            $aliasI18n = Query::getAlias($tableI18n);

            $this->innerJoin[] = array(
                $tableI18n,
                'AND' ,
                array($alias . '.' . $this->primaryKey, $aliasI18n . '.' . $this->primaryKey),
                array($aliasI18n . '.' . 'langue_id', Boutique::getInstance()->getLangue()->id)
            );
        }

        if (is_null($id) == false) {
            $this->load($id);
        }
    }

    /**
     * @return bool
     */
    public function resolveI18nTable()
    {
        if (!isset(static::$globalI18n[get_called_class()])) {
            static::$globalI18n[get_called_class()] = $this->getI18n();
        }

        return static::$globalI18n[get_called_class()];

    }

    /**
     * @return string
     */
    public function resolvePrimaryKey()
    {
        if (!isset(static::$gloabalPrimaryKey[get_called_class()])) {
            static::$gloabalPrimaryKey[get_called_class()] = $this->sql->getPrimaryKey($this->table);
        }

        return static::$gloabalPrimaryKey[get_called_class()];

    }

    /**
     * @param $id
     */
    public function load($id)
    {
        $this->hydrate($this->getOne(array(Query::getAlias($this->table) . '.' . $this->primaryKey => $id)));
    }

    /**
     * @param array $data
     */
    public function hydrate($data)
    {
        if (empty($data)) {
            // dump(debug_backtrace());
            return false;
        }

        foreach ($data as $k => $d) {
            $this->$k = $d;
        }
    }

    /**
     * @param array $formvars
     * @return array
     */
    public function get($formvars = array())
    {
        // echo $this->getQuery($formvars)->build();
        return $this->sql->getAll($this->getQuery($formvars)->build());
    }


    public function getOne($formvars = array())
    {
        return $this->sql->getOne($this->getQuery($formvars)->build());
    }

    /**
     * @param $data
     * @return mixed
     * @throws \Exception
     *
     */
    public function save($data = null)
    {
        if (property_exists($this, 'table') == false) {
            throw new DediException(
                'No property table in class ' . get_class($this)
            );
        }

        if (empty($this->primaryKey)) {
            throw new DediException(
                'No primary key defined in table ' . $this->table
            );
        }

        if (is_null($data)) {
            $data = $this->properties;
        }

        $id = $this->sql->save($this->table, $data);

        if ($this->isI18n()) {
            // TODO : save multilangue
        }

        return $id;
    }

    /**
     * @param array $formvars
     */
    public function export($formvars = array())
    {
        $result = $this->get($formvars);

        $file = SITE_DIR . 'export' . DS . 'export_' . $this->table . '_' . date('dmYHis') . '.csv' ;
        Csv::getCsvFromArray($file, $result);
    }

    /**
     * @param $file
     */
    public function import($file)
    {
        // TODO
    }

    /**
     * @return bool
     */
    public function isI18n()
    {
        return $this->i18n;
    }

    /**
     * @return bool
     *
     * check si table $this->table . "_nom" exist
     */
    private function getI18n()
    {
        $sql = "SHOW TABLES LIKE '" . $this->table . $this->suffixI18n . "'";
        $result = $this->sql->getAll($sql);
        return (!empty($result));
    }

    /**
     * @param array $formvars
     * @return Query()
     *
     * build request with table, i18n,
     * property innerJoin / leftJoin, where and options in $formvars
     */
    public function getQuery($formvars = array())
    {

        Tools::escapeDataToSql($formvars);

        $q = new Query();
        $q->select()
            ->from($this->table);

        if (!empty($this->innerJoin)) {
            foreach ($this->innerJoin as $ij) {
                $q->join('innerJoin', $ij);
            }
        }

        if (!empty($this->leftJoin)) {
            foreach ($this->leftJoin as $lj) {
                $q->join('leftJoin', $lj);
            }
        }

        if (!empty($this->where)) {
            foreach ($this->where as $w) {
                $q->where($w);
            }
        }

        $formvars = array_merge($this->extras, $formvars);

        if (!empty($formvars)) {
            foreach ($formvars as $k => $v) {
                if (method_exists($q, $k)) {
                    $q->$k($v);
                } else {
                    $q->where($k, $v);
                }
            }
        }

        return $q;
    }

    /**
     * Apply list of transformer on the data result
     *
     * @param $data
     * @return int coutn transformers apply
     */
    public function applyTranformers(&$data)
    {
        $apply = 0;
        if (!empty(static::$transformers)) {
            foreach (static::$transformers as $transformer) {
                if (!in_array('Transformer\\TransformerInterface', class_implements($transformer))) {
                    continue;
                }

                if ($this->array_depth($data) > 1) {
                    foreach ($data as $k => $d) {
                        $data[$k] = call_user_func_array(array(new $transformer, "transform"), array($d));
                    }
                } else {
                    $data = call_user_func_array(array(new $transformer, "transform"), $data);
                }
                $apply++;
            }
        }

        return $apply;
    }

    /**
     * Return properties in array
     *
     * @return array
     */
    public function toArray()
    {
        return $this->properties;
    }

    /**
     * get the db connection
     *
     * @param SiteSQL|null $newConnection
     *
     * @return SiteSQL
     */
    protected function connection(SiteSQL $newConnection = null)
    {
        static $connection;
        if ($newConnection !== null) {
            $connection = $newConnection;
        }
        return $connection;
    }

    /**
     * @param $name
     * @return SiteSQL
     */
    public function __get($name)
    {

        if ($name == 'sql') {
            return $this->connection();
        }

        if (array_key_exists($name, $this->properties)) {
            return $this->properties[$name];
        }
    }

    /**
     * @param $name
     * @param $value
     */
    public function __set($name, $value)
    {
        $this->properties[$name] = $value;
    }

    /**
     * @param $name
     * @return bool
     */
    public function __isset($name)
    {
        return !empty($this->properties[$name]);
    }

    /**
     * @param array $array
     * @return int
     *
     * return the depth of the array pass in parameter
     */
    protected function array_depth(array $array)
    {
        $max_depth = 1;

        foreach ($array as $value) {
            if (is_array($value)) {
                $depth = $this->array_depth($value) + 1;

                if ($depth > $max_depth) {
                    $max_depth = $depth;
                }
            }
        }

        return $max_depth;
    }

    /**
     * return the highest depth of instances in database
     *
     * @return bool|float
     * @throws Exception
     */
    public function getHighestDepth()
    {
        if(isset($this->properties['rang'])) return false;
        $q = new \Query();
        $q->select('MAX('.$this->table.'_rang) as max_depth')
            ->from($this->table);
        $max_depth = $this->sql->getOne($q->build());
        if($max_depth){
            return (float)$max_depth['max_depth'] + Tools::multiplyBy(1, 10);
        } else {
            return false;
        }

    }

    /**
     * return the highest depth of instances in database
     *
     * @param $params
     * @return bool|float
     * @throws Exception
     */
    public static function getStaticTableHighestDepth($table, $tableName = null)
    {
        $q = new \Query();
        $q->select('MAX('.(!is_null($tableName) ? $tableName : $table).'_rang) as max_depth')
            ->setHaveAlias(false)
            ->from($table);
        $max_depth = \SiteSQL::getInstance()->getOne($q->build());

        if($max_depth && !is_null($max_depth['max_depth'])){
            return (float)$max_depth['max_depth'] + Tools::multiplyBy(1, 10);
        } else {
            return false;
        }
    }
}
