<?php

namespace Module;

use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Router;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class Module
 * @package Module
 */
abstract class Module {

    /**
     * @var Router
     */
    protected $router;

    /**
     * @var EventDispatcherInterface
     */
    protected $eventDispatcher;

    /**
     * Module constructor.
     * @param Router $router
     * @param EventDispatcherInterface $eventDispatcher
     */
    public function __construct(Router $router, EventDispatcherInterface $eventDispatcher)
    {
        $this->eventDispatcher = $eventDispatcher;
        $this->router = $router;
    }

    /**
     * @return Router
     */
    public function getRouter()
    {
        return $this->router;
    }

    /**
     * @return EventDispatcherInterface
     */
    public function getEventDispatcher()
    {
        return $this->eventDispatcher;
    }

    /**
     * @param EventSubscriberInterface $subscriber
     * @return $this
     */
    public function addSubscriber(EventSubscriberInterface $subscriber)
    {
        $this->getEventDispatcher()->addSubscriber($subscriber);

        return $this;
    }

    /**
     * @param $dir
     * @return $this
     */
    public function addRouteCollection($dir)
    {
        // on charge les routes uniquement si cache desactivé ou cache activé et pas de cache encore généré

        if (getenv('APP_CACHE') == false || class_exists('ProjectUrlGenerator') == false) {
            $this->getRouter()->getRouter()->getRouteCollection()->addCollection(
                $this->getRouter()->getAnnotationDirectoryLoader()->load($dir)
            );
        }

        return $this;
    }

    /**
     * @return mixed
     */
    abstract public function register();
}
